/** @par LICENCE
* @verbatim
Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.
        
        ADDI-DATA GmbH 
        Dieselstrasse 3 
        D-77833 Ottersweier 
        Tel: +19(0)7223/9493-0 
        Fax: +49(0)7223/9493-92 
        http://www.addi-data-com 
        info@addi-data.com 
        
This program is free software; you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by the Free Software 
Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, write to the Free Software Foundation, 
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

You shoud also find the complete GPL in the COPYING file 
accompanying this source code.

* @endverbatim
*/

#include <linux/version.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19)
	#include <linux/config.h>
#else
	#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,33)
		#include <linux/autoconf.h>
	#else
		#include <generated/autoconf.h>
	#endif
#endif

#include <apci1016-kapi.h>
#include "apci1016-private.h"

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,27)
	#define __user 
#endif

EXPORT_NO_SYMBOLS;

static inline uint16_t Read16DigitalInput(struct pci_dev *pdev) {
//	uint16_t retval = inb(GET_BAR1(pdev)+APCI1016_DIGITAL_INPUT_REG);
//	retval |= (inb(GET_BAR1(pdev)+APCI1016_DIGITAL_INPUT_REG + 1 ) << 8);
//	return retval;

	return inw(GET_BAR1(pdev)+APCI1016_DIGITAL_INPUT_REG);
}

//------------------------------------------------------------------------------------------------------

int APCI1016_Read16DigitalInput(struct pci_dev *pdev, uint16_t* input_value)
{
	*input_value = Read16DigitalInput(pdev);
	return 0;
}

//------------------------------------------------------------------------------------------------------

int do_CMD_APCI1016_Read16DigitalInput(struct pci_dev *pdev, unsigned int cmd, unsigned long arg)
{
	uint16_t input_value = Read16DigitalInput(pdev);
		
	/* returns value to the user */
	if ( copy_to_user( (uint16_t __user *) arg, (uint16_t*) &input_value, sizeof(input_value) ) )
		return -EFAULT;
	
	return 0;
}
