/** @par LICENCE
* @verbatim
 Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code and the documentation.

        ADDI-DATA GmbH
        Dieselstrasse 3
        D-77833 Ottersweier
        Tel: +19(0)7223/9493-0
        Fax: +49(0)7223/9493-92
        http://www.addi-data-com
        info@addi-data.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You shoud find the complete GPL in the COPYING file accompanying
    this source code.
* @endverbatim
*/

#include "apci1016-private.h"
#include "vtable.h"

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
#include <linux/sysfs.h>
#endif

MODULE_LICENSE("GPL");
MODULE_AUTHOR("ADDI-DATA GmbH <info@addi-data.com>");
MODULE_DESCRIPTION("apci1016 IOCTL driver");

EXPORT_NO_SYMBOLS;

#ifndef __devinit
#define __devinit
#define __devexit
#define __devinitdata
#define __devexit_p
#endif

//------------------------------------------------------------------------------
atomic_t apci1016_count = ATOMIC_INIT(0);
unsigned int apci1016_majornumber = 0;

//------------------------------------------------------------------------------
static int __devinit apci1016_probe_one(struct pci_dev *dev,const struct pci_device_id *ent);
static void __devexit apci1016_remove_one(struct pci_dev *dev);

//------------------------------------------------------------------------------
/** The ID table is an array of struct pci_device_id ending with a all-zero entry.
Each entry consists of:

	- vendor, device	Vendor and device ID to match (or PCI_ANY_ID)
	- subvendor,	Subsystem vendor and device ID to match (or PCI_ANY_ID)
	- subdevice class,		Device class to match. The class_mask tells which bits
	- class_mask	of the class are honored during the comparison.
	- driver_data	Data private to the driver.
*/

static struct pci_device_id apci1016_pci_tbl[] __devinitdata = {
	{
		APCI1016_BOARD_VENDOR_ID,
		APCI1016_BOARD_DEVICE_ID,
		PCI_ANY_ID,
		PCI_ANY_ID,
		0,
		0,
		0
	},

	{ 0 },	/* terminate list */
};

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
	#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,15)
		static struct class_simple * apci1016_class = NULL;
	#else
		static struct class * apci1016_class = NULL;
	#endif
#endif

MODULE_DEVICE_TABLE (pci, apci1016_pci_tbl);

//----------------------------------------------------------------------------------------------------------------------
static struct pci_driver apci1016_pci_driver = {
	.name		= __DRIVER_NAME,
	.probe		= apci1016_probe_one,
	.remove		= __devexit_p(apci1016_remove_one),
	.id_table	= apci1016_pci_tbl,
};
//------------------------------------------------------------------------------
static struct file_operations apci1016_fops =
{
	#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,36)

        .ioctl					= apci1016_ioctl_lookup,
   #else
        .unlocked_ioctl         = apci1016_ioctl_lookup,
   #endif
	.open		= apci1016_open_lookup,
	.release	= apci1016_release_lookup,
 /*
  * If using signal to inform user mode,
  * uncomment following line:
  */
  //.fasync		= apci1016_fasync_lookup,
};

//-------------------------------------------------------------------
/** event: a new card is detected.
 *
 * Historically - in former ADDI-DATA drivers - data about a board has been stored
 * in a static structure.
 * This led to huge duplication of information since most of these information
 * are already present in the pci_dev struct.
 *
 * Now we manipulate this standard structure provided by the OS and we attach
 * private data using the field driver_data, if necessary.
 *
 *  */
static int __devinit apci1016_probe_one(struct pci_dev *dev, const struct pci_device_id *ent)
{
	{
		int ret = pci_enable_device(dev);
		if (ret)
		{
			printk(KERN_ERR "%s: pci_enable_device failed\n",__DRIVER_NAME);
			return ret;
		}
	}
	// needed to enable PCI DMA
	pci_set_master(dev);

	/* 2.4 only : check_region is deprecated in 2.6 */
	#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
		/* check if BAR 0 is free */
		if ( check_region(dev->resource[0].start, pci_resource_len(dev,0)) )
		{
			printk(KERN_WARNING "%s: BAR 0 (%lX) is already in use\n",__DRIVER_NAME,dev->resource[0].start);
			return -EBUSY;
		}
		/* check if BAR 1 is free */
		if ( check_region(dev->resource[1].start, pci_resource_len(dev,1)) )
		{
			printk(KERN_WARNING "%s: BAR 1 (%lX) is already in use\n",__DRIVER_NAME,dev->resource[1].start);
			return -EBUSY;
		}
	#endif // 2.4

		/* allocate a new data structure containing board private data */
		{
			struct apci1016_str_BoardInformations * newboard_data = NULL;
			newboard_data = kmalloc( sizeof( struct apci1016_str_BoardInformations) , GFP_ATOMIC);
			if (!newboard_data)
			{
				printk(KERN_CRIT "Can't allocate memory for new board %s\n",pci_name(dev));
				return -ENOMEM;
			}
			/* link standard data structure to the board's private one */
			pci_set_drvdata(dev,newboard_data);

			apci1016_init_priv_data(newboard_data);
		}

	/* lock BAR IO ports ressources */
	{
		int ret = pci_request_regions(dev,__DRIVER_NAME);
		if (ret)
		{
			printk(KERN_ERR "%s: pci_request_regions failed\n",__DRIVER_NAME);
			return ret;
		}
	}

	{
		/* increase the global board count */
		atomic_inc(&apci1016_count);
		printk(KERN_INFO "%s: board %s managed (minor number will be %d) \n",__DRIVER_NAME, pci_name(dev), atomic_read(&apci1016_count)-1);
	}

	/* create /proc entry */
	apci1016_proc_create_device(dev, atomic_read(&apci1016_count)-1);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
	while(1)
	{
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,26)
	struct class_device *cdev = NULL;
#else // >= 2.6.26
	struct device *cdev = NULL;
#endif //2.6.26

	int minor = (atomic_read(&apci1016_count)-1);

		/* don't execute if class not exists */
		if (IS_ERR(apci1016_class))
				break;

		#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,15)
			cdev = class_simple_device_add (apci1016_class, MKDEV(apci1016_majornumber, minor), NULL, "%s_%d", __DRIVER_NAME, minor);
		#elif LINUX_VERSION_CODE < KERNEL_VERSION(2,6,26)
			cdev = class_device_create (apci1016_class, NULL, MKDEV(apci1016_majornumber, minor), NULL, "%s_%d", __DRIVER_NAME, minor);
		#elif LINUX_VERSION_CODE < KERNEL_VERSION(2,6,27)
			cdev = device_create (apci1016_class, NULL, MKDEV(apci1016_majornumber, minor), "%s_%d", __DRIVER_NAME, minor);
		#else
			cdev = device_create (apci1016_class, NULL, MKDEV(apci1016_majornumber, minor), NULL, "%s_%d", __DRIVER_NAME, minor);
		#endif

		if (IS_ERR(cdev))
		{
			printk (KERN_WARNING "%s: class_device_create error\n", __DRIVER_NAME);
		}
		break;
	}
#endif

	return 0;
}


//------------------------------------------------------------------------------
/** This function registers the driver with register_chrdev.
 *
 * @todo For kernel 2.6, use udev
 *  */

static int apci1016_register(void)
{

	//Registration of driver
	apci1016_majornumber = register_chrdev(0, __DRIVER_NAME, &apci1016_fops);

	if (apci1016_majornumber < 0)
	{
		printk (KERN_ERR "%s: register_chrdev returned %d ... aborting\n",__DRIVER_NAME,apci1016_majornumber);
		return -ENODEV;
	}
	return 0;
}

//------------------------------------------------------------------------------
/** Called when module loads. */
static int __init apci1016_init(void)
{

	apci1016_init_vtable(apci1016_vtable);

	if( apci1016_register() )
	{
		return -ENODEV;
	}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,15)
	apci1016_class = class_simple_create (THIS_MODULE, __DRIVER_NAME);
#else
	apci1016_class = ADDI_CLASS_CREATE(THIS_MODULE, __DRIVER_NAME);
#endif

	if (IS_ERR(apci1016_class))
	{
		printk (KERN_WARNING "%s: class_create error\n",__DRIVER_NAME );
	}
#endif

	/* registred, now create root /proc entry */
	apci1016_proc_init();

	printk(KERN_INFO "%s: loaded\n",__DRIVER_NAME);

	/* now, subscribe to PCI bus subsystem  */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,18)
	/** @note The module will load anyway even if the call to pci_module_init() fails */
	if ( pci_module_init (&apci1016_pci_driver) != 0)
#else
	if ( pci_register_driver (&apci1016_pci_driver) != 0)
#endif
	{
		printk("%s: no device found\n",__DRIVER_NAME);
		apci1016_pci_driver.name = "";
	}

	return 0;
}

//-------------------------------------------------------------------
/** event: a card is removed (also called when module is unloaded) */
static void __devexit apci1016_remove_one(struct pci_dev *dev)
{

	/* do OS-dependant thing we don't really want to know of :) */
	pci_disable_device(dev);

	/* deallocate BAR IO Ports ressources */
	pci_release_regions(dev);

	/* delete associated /proc entry */
	apci1016_proc_release_device(dev);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
	while(1)
	{
		int minor = (atomic_read(&apci1016_count)-1);

		/* don't execute if class not exists */
		if(IS_ERR(apci1016_class))
				break;

		#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,26)
			#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,15)
				class_simple_device_remove (MKDEV(apci1016_majornumber, minor));
			#else
				class_device_destroy (apci1016_class, MKDEV(apci1016_majornumber, minor));
			#endif
		#else // >= 2.6.26
			device_destroy (apci1016_class, MKDEV(apci1016_majornumber, minor));
		#endif // 2.6.26

		break;
	}
#endif

	/* free private device data*/
	if (APCI1016_PRIVDATA(dev))
		kfree(APCI1016_PRIVDATA(dev));

	/* decrement global board count*/
	atomic_dec(&apci1016_count);

	printk("%s: board %s unloaded \n",__DRIVER_NAME,pci_name(dev));

}

//------------------------------------------------------------------------------
/** This function unregisters the driver with unregister_chrdev.
 *
 * @todo For kernel 2.6, use udev
 *  */

static void apci1016_unregister(void)
{
	unregister_chrdev(apci1016_majornumber, __DRIVER_NAME);
}

//-------------------------------------------------------------------
/** Called when module is unloaded. */
static void __exit apci1016_exit(void)
{

	/* unsubscribe to PCI bus subsystem */
	if (apci1016_pci_driver.name[0])
	{
		pci_unregister_driver (&apci1016_pci_driver);
	}

	/* unsubscribe to /dev VFS */
	apci1016_unregister();

	/* delete /proc root */
	apci1016_proc_release();

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
	#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,15)
		class_simple_destroy (apci1016_class);
	#else
		class_destroy (apci1016_class);
	#endif
#endif
}
//------------------------------------------------------------------------------
module_exit(apci1016_exit);
module_init(apci1016_init);
