/** @file ioctl.c
*
* @author Sylvain Nahas
*
* This module implements the apci1016_do_ioctl function
*/

/** @par LICENCE
* @verbatim
 Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code and the documentation.
        
        ADDI-DATA GmbH
        Dieselstrasse 3 
        D-77833 Ottersweier
        Tel: +19(0)7223/9493-0
        Fax: +49(0)7223/9493-92
        http://www.addi-data-com
        info@addi-data.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. 
        
    You shoud find the complete GPL in the COPYING file accompanying 
    this source code. 
* @endverbatim
*/

#include <linux/version.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(4,12,0)
    #include <linux/uaccess.h>
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19)
	#include <linux/config.h>
#else
	#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,33)
		#include <linux/autoconf.h>
	#else
		#include <generated/autoconf.h>
	#endif
#endif

// #include <linux/module.h>
// #include <linux/fs.h>
// #include <linux/pci.h>
// #include <asm/io.h>
// #include <asm/system.h>
// #include <asm/uaccess.h>
// #include <asm/bitops.h>
// #include <linux/sched.h>

// #include <linux/interrupt.h>
// #include <linux/kernel.h>
// #include <linux/delay.h>

#include "apci1016-private.h"
#include "vtable.h"

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,27)
	#define __user 
#endif

//------------------------------------------------------------------------------
/** dummy command to be called when a ioctl command is incorrect */
static int apci1016_do_dummy(struct pci_dev * pdev, unsigned int cmd, unsigned long arg)
{
	printk (KERN_WARNING "%s: %d: invalid ioctl\n",__DRIVER_NAME,_IOC_NR(cmd));
	return -EINVAL;
}
//------------------------------------------------------------------------------
/** add new ioctl handlers here */
void apci1016_init_vtable(vtable_t vtable)
{		
	apci1016_dummyze_vtable(vtable, apci1016_do_dummy);
	

	__APCI1016_DECLARE_IOCTL_HANDLER(vtable,CMD_APCI1016_CheckAndGetPCISlotNumber,do_CMD_APCI1016_CheckAndGetPCISlotNumber);
	__APCI1016_DECLARE_IOCTL_HANDLER(vtable,CMD_APCI1016_GetHardwareInformation,do_CMD_APCI1016_GetHardwareInformation);	
	__APCI1016_DECLARE_IOCTL_HANDLER(vtable,CMD_APCI1016_SetBoardInformation,do_CMD_APCI1016_SetBoardInformation);
	__APCI1016_DECLARE_IOCTL_HANDLER(vtable,CMD_APCI1016_Read16DigitalInput,do_CMD_APCI1016_Read16DigitalInput);
}

//------------------------------------------------------------------------------
int apci1016_do_ioctl(struct pci_dev * pdev,unsigned int cmd,unsigned long arg)
{

	/* boundaries check 
	 * 
	 * VTABLE_ELEMENT_NB(vtable_t) = __APCI1016_UPPER_IOCTL_CMD +1
	 * maximum index value = __APCI1016_UPPER_IOCTL_CMD = VTABLE_ELEMENT_NB(vtable_t) -1
	 * 
	 * the idea here is to favorize compilation-time 
	 * 
	 * */
	
	if (_IOC_NR(cmd) > (VTABLE_ELEMENT_NB(vtable_t)-1) )
		return apci1016_do_dummy(pdev,cmd,arg);
 
	/* call actual ioctl handler - should be safe now */
	return (apci1016_vtable[_IOC_NR(cmd)]) (pdev, cmd, arg);
}
//------------------------------------------------------------------------------
int do_CMD_APCI1016_CheckAndGetPCISlotNumber(struct pci_dev * pdev, unsigned int cmd, unsigned long arg)
{
	int retval = 0; /* return value of this call is the number of boards */
	
    /* this command returns an area of size CONFIG_APCI1016_MAX_BOARD_NBR+1 filled with the PCI_SLOT() value and as last field the major number associated to the device - this is deprecated and should not be used anymore.
	The call returns 0 when memory is too low or the number of boards  */	
   
	char * b_SlotArray = kmalloc(CONFIG_APCI1016_MAX_BOARD_NBR+1 * sizeof(char),GFP_KERNEL);
    if (!b_SlotArray)
	{
		/* bad param. Let output some information for debug */
		printk("%s: CMD_APCI1016_CheckAndGetPCISlotNumber: -EINVAL\n",__DRIVER_NAME);
		return 0;
	}
	    
	memset(b_SlotArray,0,CONFIG_APCI1016_MAX_BOARD_NBR+1 );

	printk("%s: the use of CMD_APCI1016_CheckAndGetPCISlotNumber is deprecated\n",__DRIVER_NAME);

	/* record the PCI_SLOT for each device from 0 to CONFIG_APCI1016_MAX_BOARD_NBR  */
	{	    	
		int i;
		for(i = 0; ( ( i < atomic_read(&apci1016_count) ) || ( i< CONFIG_APCI1016_MAX_BOARD_NBR ) ); i++)
		{
			
			struct pci_dev * dev =  apci1016_lookup_board_by_index(i);
			
			if (!dev) break;
			
			b_SlotArray[i] = PCI_SLOT(dev->devfn);
			
		}
    }
    b_SlotArray[CONFIG_APCI1016_MAX_BOARD_NBR] = apci1016_majornumber;

    if ( copy_to_user ( (uint8_t __user*)arg, b_SlotArray, sizeof (uint8_t[CONFIG_APCI1016_MAX_BOARD_NBR+1])) )
	{
    	/* bad address. Let output some information for debug */
    	printk("%s: CMD_APCI1016_CheckAndGetPCISlotNumber: -EFAULT\n",__DRIVER_NAME);
    	kfree(b_SlotArray);
    	return 0;
	}
	/* return the smaller value between CONFIG_APCI1016_MAX_BOARD_NBR and number of board - Note: apci1016_count is assumed to to be always consistent with the system PCI devices list */
    retval = ( atomic_read(&apci1016_count) < CONFIG_APCI1016_MAX_BOARD_NBR ? atomic_read(&apci1016_count) : CONFIG_APCI1016_MAX_BOARD_NBR );

	kfree(b_SlotArray);	            	
	
	return retval;	

}



//------------------------------------------------------------------------------
/** Returns the informations of base address, IRQ to the user.
 * @deprecated: use-space doesn't need it and kernel-space has better means to get it.
 *  */
int do_CMD_APCI1016_GetHardwareInformation(struct pci_dev * pdev, unsigned int cmd, unsigned long arg)
{
	str_BaseInformations s_BaseInformations;
	s_BaseInformations.ui_BaseAddress[0] = GET_BAR0(pdev);
	s_BaseInformations.ui_BaseAddress[1] = GET_BAR1(pdev);
	s_BaseInformations.ui_BaseAddress[2] = 0;
	s_BaseInformations.ui_BaseAddress[3] = 0;
	s_BaseInformations.ui_BaseAddress[4] = 0;
	s_BaseInformations.b_Interrupt = pdev->irq;
	s_BaseInformations.b_SlotNumber = PCI_SLOT(pdev->devfn);

    if ( copy_to_user ((str_BaseInformations __user *)arg,&s_BaseInformations, sizeof(s_BaseInformations)) )
		return -EFAULT;
	
	return 0;
}

//------------------------------------------------------------------------------
int do_CMD_APCI1016_SetBoardInformation(struct pci_dev * pdev, unsigned int cmd, unsigned long arg)
{
	return 0;
}
